package svc

import (
	"applet/app/admin/md"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/enum"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"github.com/360EntSecGroup-Skylar/excelize"
	"strconv"
	"time"
)

func NursingHomeDataStatisticsExport(req md.NursingHomeDataStatisticsExportReq) {
	var titleList []string
	var fileName string
	var now = time.Now()
	xlsx := excelize.NewFile()
	xlsx.SetSheetRow("Sheet1", "A1", &titleList)
	switch req.Kind {
	case 1:
		//订单列表
		fileName = "/export/nursing_home/订单列表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"

		//查找订单数据
		sess := db.Db.Desc("nursing_home_package_ord.id")
		if req.EnterpriseId != 0 {
			sess.And("nursing_home_package_ord.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("nursing_home_package_ord.create_at >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("nursing_home_package_ord.create_at <= ?", req.EndDate)
		}
		var m []*db.NursingHomePackageOrdWithUserIdentity
		_, err := sess.
			Join("LEFT", "user_identity", "nursing_home_package_ord.user_identity_id = user_identity.id").
			Join("LEFT", "enterprise", "enterprise.id = nursing_home_package_ord.enterprise_id").
			Join("LEFT", "user", "user.id = nursing_home_package_ord.uid").
			FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<NursingHomeDataStatisticsExport1>>>>>Error:::", err.Error())
			return
		}
		titleList = []string{"单位名称", "用户身份名称", "用户手机号", "购买类型", "总价", "商户订单号", "支付状态", "购买时间"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var kind string //1:按年购买 2:按月购买 3:按天购买 4:补餐
			switch v.NursingHomePackageOrd.Kind {
			case 1:
				kind = "按年购买"
				break
			case 2:
				kind = "按月购买"
				break
			case 3:
				kind = "按天购买"
				break
			case 4:
				kind = "补餐"
				break
			}

			var state string //0:待支付 1:支付成功 2:支付失败
			switch v.NursingHomePackageOrd.State {
			case enum.NursingHomePackageOrdStateForWait:
				state = enum.NursingHomePackageOrdState.String(enum.NursingHomePackageOrdStateForWait)
				break
			case enum.NursingHomePackageOrdStateForSuccess:
				state = enum.NursingHomePackageOrdState.String(enum.NursingHomePackageOrdStateForWait)
				break
			case enum.NursingHomePackageOrdStateForFail:
				state = enum.NursingHomePackageOrdState.String(enum.NursingHomePackageOrdStateForFail)
				break
			}
			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.Enterprise.Name, v.NursingHomePackageOrd.UserIdentityName, v.User.Phone, kind, v.NursingHomePackageOrd.TotalPrice,
				v.NursingHomePackageOrd.OutTradeNo, state, v.NursingHomePackageOrd.CreateAt})
			j++
		}
		xlsx.SaveAs("./static" + fileName)
		break
	case 2:
		//退款申请表
		fileName = "/export/nursing_home/退款申请表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"
		var m []*db.NursingHomeUserRefundDayWithData
		sess := db.Db.Desc("nursing_home_user_refund_day.id")
		if req.EnterpriseId != 0 {
			sess.And("enterprise.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("nursing_home_user_refund_day.create_at >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("nursing_home_user_refund_day.create_at <= ?", req.EndDate)
		}
		_, err := sess.
			Join("LEFT", "nursing_home_user_with_day", "nursing_home_user_refund_day.records_id = nursing_home_user_with_day.id").
			Join("LEFT", "nursing_home_package_ord", "nursing_home_user_refund_day.out_trade_no = nursing_home_package_ord.out_trade_no").
			Join("LEFT", "user_identity", "nursing_home_user_with_day.identity_id = user_identity.id").
			Join("LEFT", "user", "user_identity.uid = user.id").
			Join("LEFT", "enterprise", "user_identity.enterprise_id = enterprise.id").
			Desc("nursing_home_user_refund_day.id").
			FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<NursingHomeDataStatisticsExport2>>>>>Error:::", err.Error())
			return
		}

		titleList = []string{"退款单号", "所属订单号", "单位名称", "用户身份名称", "用户手机号", "就餐类型", "就餐时间", "金额", "状态", "退款时间"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var kind string //就餐类型(1:早餐 2:午餐 3:晚餐)
			switch v.NursingHomeUserWithDay.Kind {
			case enum.NursingHomeUserWithDayKindForBreakfast:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForBreakfast)
				break
			case enum.NursingHomeUserWithDayKindForLunch:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForLunch)
				break
			case enum.NursingHomeUserWithDayKindForDinner:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForDinner)
				break
			}

			var state string //状态(1:审核中 2:审核通过 3:审核拒绝 4:退款已完成)
			switch v.NursingHomeUserRefundDay.State {
			case enum.NursingHomeUserRefundDayStateForAuditing:
				state = enum.NursingHomeUserRefundDayState.String(enum.NursingHomeUserRefundDayStateForAuditing)
				break
			case enum.NursingHomeUserRefundDayStateForAuditPass:
				state = enum.NursingHomeUserRefundDayState.String(enum.NursingHomeUserRefundDayStateForAuditPass)
				break
			case enum.NursingHomeUserRefundDayStateForAuditReject:
				state = enum.NursingHomeUserRefundDayState.String(enum.NursingHomeUserRefundDayStateForAuditReject)
				break
			case enum.NursingHomeUserRefundDayStateForAuditComplete:
				state = enum.NursingHomeUserRefundDayState.String(enum.NursingHomeUserRefundDayStateForAuditComplete)
				break
			}
			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.NursingHomeUserRefundDay.OutRequestNo, v.NursingHomeUserRefundDay.OutTradeNo, v.Enterprise.Name,
				v.NursingHomePackageOrd.UserIdentityName, v.User.Phone, kind, v.NursingHomeUserWithDay.Date, v.NursingHomeUserRefundDay.Amount, state, v.NursingHomeUserRefundDay.RefundDate})
			j++
		}
		xlsx.SaveAs("./static" + fileName + ".xlsx")
		break
	case 3:
		//预定统计表
		fileName = "/export/nursing_home/预定统计表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"
		var m []*db.NursingHomePackageOrdForReserveWithUserIdentity
		sess := db.Db.Desc("nursing_home_package_ord_for_reserve.id")
		if req.EnterpriseId != 0 {
			sess.And("enterprise.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("nursing_home_package_ord_for_reserve.meal_time_start >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("nursing_home_package_ord_for_reserve.meal_time_start <= ?", req.EndDate)
		}
		_, err := sess.
			Join("LEFT", "nursing_home_package_ord", "nursing_home_package_ord_for_reserve.out_trade_no = nursing_home_package_ord.out_trade_no").
			Join("LEFT", "user_identity", "nursing_home_package_ord.user_identity_id = user_identity.id").
			Join("LEFT", "enterprise", "enterprise.id = user_identity.enterprise_id").
			Join("LEFT", "user", "user.id = user_identity.uid").FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<NursingHomeDataStatisticsExport2>>>>>Error:::", err.Error())
			return
		}

		titleList = []string{"所属订单号", "单位名称", "用户身份名称", "用户手机号", "预定类型", "预定月份", "就餐时间-开始", "就餐时间-截止"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var kind string //预定类型(1:早餐 2:午餐 3:晚餐)
			switch v.NursingHomePackageOrdForReserve.Kind {
			case enum.NursingHomeUserWithDayKindForBreakfast:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForBreakfast)
				break
			case enum.NursingHomeUserWithDayKindForLunch:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForLunch)
				break
			case enum.NursingHomeUserWithDayKindForDinner:
				kind = enum.NursingHomeUserWithDayKind.String(enum.NursingHomeUserWithDayKindForDinner)
				break
			}

			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.NursingHomePackageOrdForReserve.OutTradeNo, v.Enterprise.Name, v.NursingHomePackageOrd.UserIdentityName,
				v.User.Phone, kind, v.NursingHomePackageOrdForReserve.ReserveMonth, v.NursingHomePackageOrdForReserve.MealTimeStart, v.NursingHomePackageOrdForReserve.MealTimeEnd})
			j++
		}
		xlsx.SaveAs("./static" + fileName + ".xlsx")
		break
	}
	//新增数据
	nursingHomeExportRecordsDb := db.NursingHomeExportRecordsDb{}
	nursingHomeExportRecordsDb.Set()
	marshal, _ := json.Marshal(req)
	nursingHomeExportRecordsDb.NursingHomeExportRecordsInsert(&model.NursingHomeExportRecords{
		Name:         fileName,
		DownloadPath: fileName,
		Kind:         req.Kind,
		ReqContent:   string(marshal),
		CreateAt:     now.Format("2006-01-02 15:04:05"),
		UpdateAt:     now.Format("2006-01-02 15:04:05"),
	})
}

func SelfSupportForSchoolDataStatisticsExport(req md.SelfSupportForSchoolDataStatisticsExportReq) {
	var titleList []string
	var fileName string
	var now = time.Now()
	xlsx := excelize.NewFile()
	xlsx.SetSheetRow("Sheet1", "A1", &titleList)
	switch req.Kind {
	case 1:
		//学生列表
		fileName = "/export/self_support_for_school/学生列表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"

		//查找订单数据
		sess := db.Db.Desc("user_identity.id")
		sess.And("identity =?", enum.UserIdentityForSelfSupportForStudent)
		if req.EnterpriseId != 0 {
			sess.And("user_identity.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("user_identity.create_at >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("user_identity.create_at <= ?", req.EndDate)
		}
		var m []*db.UserIdentityWithUser
		_, err := sess.
			Join("LEFT", "user", "user_identity.uid = user.id").
			Join("LEFT", "enterprise", "user_identity.enterprise_id = enterprise.id").
			Join("LEFT", "class_with_user", "class_with_user.user_identity_id = user_identity.id").
			Join("LEFT", "class", "class_with_user.class_id = class.id").
			Join("LEFT", "grade", "class.grade_id = grade.id").
			Join("LEFT", "self_support_for_user_face_info", "user_identity.id = self_support_for_user_face_info.user_identity_id").
			FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<SelfSupportForSchoolDataStatisticsExport1>>>>>Error:::", err.Error())
			return
		}
		titleList = []string{"单位名称", "姓名", "身份证号", "家长电话", "采脸方式", "支付状态", "刷脸状态", "年级", "班级", "注册时间"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var kind string //采集人脸方式(1:个采 2:集采)
			switch v.SelfSupportForUserFaceInfo.CollectFaceType {
			case 1:
				kind = "个采"
				break
			case 2:
				kind = "集采"
				break
			}

			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.Enterprise.Name, v.UserIdentity.Name, v.UserIdentity.IdNo, v.User.Phone, kind,
				v.SelfSupportForUserFaceInfo.SchoolFacePaymentStatus, v.SelfSupportForUserFaceInfo.SchoolFacePassStatus, v.Grade.Name, v.Class.Name, v.UserIdentity.CreateAt})
			j++
		}
		xlsx.SaveAs("./static" + fileName)
		break
	case 2:
		//教师列表
		fileName = "/export/self_support_for_school/教师列表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"

		//查找订单数据
		sess := db.Db.Desc("user_identity.id")
		sess.And("identity =?", enum.UserIdentityForSelfSupportForTeacher)
		if req.EnterpriseId != 0 {
			sess.And("user_identity.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("user_identity.create_at >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("user_identity.create_at <= ?", req.EndDate)
		}
		var m []*db.UserIdentityWithUser
		_, err := sess.
			Join("LEFT", "user", "user_identity.uid = user.id").
			Join("LEFT", "enterprise", "user_identity.enterprise_id = enterprise.id").
			Join("LEFT", "self_support_for_user_face_info", "user_identity.id = self_support_for_user_face_info.user_identity_id").
			FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<SelfSupportForSchoolDataStatisticsExport2>>>>>Error:::", err.Error())
			return
		}
		titleList = []string{"单位名称", "姓名", "身份证号", "电话", "采脸方式", "支付状态", "刷脸状态", "注册时间"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var kind string //采集人脸方式(1:个采 2:集采)
			switch v.SelfSupportForUserFaceInfo.CollectFaceType {
			case 1:
				kind = "个采"
				break
			case 2:
				kind = "集采"
				break
			}

			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.Enterprise.Name, v.UserIdentity.Name, v.UserIdentity.IdNo, v.User.Phone, kind,
				v.SelfSupportForUserFaceInfo.SchoolFacePaymentStatus, v.SelfSupportForUserFaceInfo.SchoolFacePassStatus, v.UserIdentity.CreateAt})
			j++
		}
		xlsx.SaveAs("./static" + fileName)
		break
	case 3:
		//消费明细表
		fileName = "/export/self_support_for_school/消费明细表_" + utils.Int64ToStr(now.UnixMilli()) + ".xlsx"
		var m []*db.SelfSupportForSchoolOrdWithUserIdentity
		sess := db.Db.Desc("self_support_for_school_ord.id")
		if req.EnterpriseId != 0 {
			sess.And("self_support_for_school_ord.enterprise_id =?", req.EnterpriseId)
		}
		if req.StartDate != "" {
			sess.And("self_support_for_school_ord.face_time >= ?", req.StartDate)
		}
		if req.EndDate != "" {
			sess.And("self_support_for_school_ord.face_time <= ?", req.EndDate)
		}
		_, err := sess.
			Join("LEFT", "user_identity", "self_support_for_school_ord.user_identity_id = user_identity.id").
			Join("LEFT", "enterprise", "enterprise.id = user_identity.enterprise_id").
			Join("LEFT", "user", "user.id = user_identity.uid").
			Join("LEFT", "class_with_user", "class_with_user.user_identity_id = user_identity.id").
			Join("LEFT", "class", "class_with_user.class_id = class.id").
			Join("LEFT", "grade", "class.grade_id = grade.id").
			FindAndCount(&m)
		if err != nil {
			logx.Error(err)
			println("<<<<SelfSupportForSchoolDataStatisticsExport3>>>>>Error:::", err.Error())
			return
		}

		titleList = []string{"订单编号", "支付交易号", "用户手机号", "就餐人姓名", "学校名称", "年级", "班级", "金额", "支付状态", "就餐时间", "设备编号", "身份"}
		j := 2 //表头被第一行用了，只能从第二行开始
		for _, v := range m {
			var state string //支付状态(0:待支付 1:支付成功)
			switch v.SelfSupportForSchoolOrd.OrderStatus {
			case 0:
				state = "待支付"
				break
			case 1:
				state = "支付成功"
				break
			}

			var identity string
			switch enum.UserIdentityForSelfSupportForTeacher {
			case enum.UserIdentityForSelfSupportForTeacher:
				identity = enum.UserIdentity.String(enum.UserIdentityForSelfSupportForTeacher)
				break
			case enum.UserIdentityForSelfSupportForStudent:
				identity = enum.UserIdentity.String(enum.UserIdentityForSelfSupportForStudent)
				break
			default:
				identity = "未知"
			}

			xlsx.SetSheetRow("Sheet1", "A"+strconv.Itoa(j), &[]interface{}{v.SelfSupportForSchoolOrd.OutOrderNo, v.SelfSupportForSchoolOrd.TradeNo, v.UserIdentity.Name, v.Enterprise.Name,
				v.Grade.Name, v.Class.Name, v.SelfSupportForSchoolOrd.TradeAmount, state, v.SelfSupportForSchoolOrd.FaceTime, v.SelfSupportForSchoolOrd.DeviceSn, identity})
			j++
		}
		xlsx.SaveAs("./static" + fileName + ".xlsx")
		break
	}
	//新增数据
	selfSupportForSchoolExportRecordsDb := db.SelfSupportForSchoolExportRecordsDb{}
	selfSupportForSchoolExportRecordsDb.Set()
	marshal, _ := json.Marshal(req)
	selfSupportForSchoolExportRecordsDb.SelfSupportForSchoolExportRecordsInsert(&model.SelfSupportForSchoolExportRecords{
		Name:         fileName,
		DownloadPath: fileName,
		Kind:         req.Kind,
		ReqContent:   string(marshal),
		CreateAt:     now.Format("2006-01-02 15:04:05"),
		UpdateAt:     now.Format("2006-01-02 15:04:05"),
	})
}
